function Iterator(resource) {
	this.res = resource;
	this.index = 0;
	this.curr = 0;

	this.hasNext = function() {
		return this.index < this.res.length;
	}
	this.next = function() {
		this.curr = this.res[this.index++];
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function ConstIterator(value) {
	this.curr = value;

	this.hasNext = function() {
		return true;
	}
	this.next = function() {
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function IntsIterator(from, to) {
	this.curr = from;
	this.end = to;

	this.hasNext = function() {
		return this.curr <= this.end;
	}
	this.next = function() {
		return this.curr++;
	}
	this.peek = function() {
		return this.curr;
	}
}

function IdentityIterator(iterator, functor, data) {
	this.inner = iterator;

	this.hasNext = function() {
		return this.inner.hasNext();
	}
	this.next = function() {
		return this.inner.peek(); // YES - peek()
	}
	this.peek = function() {
		return this.inner.peek();
	}
}

function MapIterator(iterator, functor, data) {
	this.inner = iterator;
	this.curr = 0;

	this.hasNext = function() {
		return this.inner.hasNext();
	}
	this.next = function() {
		this.curr = functor(this.inner.next(), data);
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function Series(resource) {
	this.res = resource;

	this.getIterator = function() {
		return new Iterator(this.res);
	}
}

function ConstSeries(resource) {
	this.res = resource;

	this.getIterator = function() {
		return new ConstIterator(this.res);
	}
}

function IntsSeries(from, to) {
	this.from = from;
	this.to = to;

	this.getIterator = function() {
		return new IntsIterator(this.from, this.to);
	}
}


/**
 * Dependency series...
 **/

function IteratingSeries(series) {
	this.base = series;
	this.idx = -1;

	this.getIterator = function() {
		this.idx = -1;

		return new MapIterator(this.base.getIterator(), 
		                       function(v, series) { series.idx++; return v; },
		                       this);
	}
	this.getIndex = function() {
		return this.idx;
	}
}

function ControlledSeries(series, baseSeries) {
	this.ser = series;
	this.base = baseSeries;

	this.getIterator = function() {
		// FIXME: HACK accessing the series resource directly (needs redesign)
		if (this.ser.res[this.base.getIndex()] == undefined) {
			alert("Undef: @" + this.base.getIndex() + " : " + this.ser.res
			      + " : " + this.base.res);
			return null;
		}
		return this.ser.res[this.base.getIndex()].getIterator();
	}
}


/**
 * Metric inspection.
 **/

function inspect(it) {
	if (! it.hasNext()) {
		return { min: 0, max: 0, sum: 0, count: 0 };
	}
		
	var min = it.next();
	var max = min;
	var sum = min;
	var count = 1;
	while(it.hasNext()) {
		var e = it.next();
		sum += e;
		min = min<e ? min:e;
		max = max>e ? max:e;
		count++;
	}

	return { min: min, max: max, sum: sum, count: count };
}


// FIXME: so far unused...

function NeighborSumIterator(iterator) {
	return new MapIterator(iterator,
			function(v, data) {
		var r = v + data.prev;
		data.prev = v;
		return r;
	}, {
		prev :iterator.next()
	});
}

